package org.implab.gradle.containers.tasks;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.gradle.api.Action;
import org.gradle.api.provider.ListProperty;
import org.gradle.api.provider.Property;
import org.gradle.api.tasks.Input;
import org.gradle.api.tasks.Optional;
import org.implab.gradle.containers.ImageName;

public abstract class RunImage extends DockerCliTask {

    public final String RUN_COMMAND = "run";

    @Input
    @Optional
    public abstract ListProperty<String> getOptions();

    @Input
    public abstract Property<ImageName> getImageName();

    @Input
    @Optional
    public abstract ListProperty<String> getCommandLine();

    @Override
    protected java.util.Optional<String> getSubCommand() {
        return java.util.Optional.of(RUN_COMMAND);
    }

    @Override
    protected Collection<String> getSubCommandArguments() {
        List<String> args = new ArrayList<String>();

        args.addAll(getOptions().get());
        args.add(getImageName().get().toString());
        args.addAll(getCommandLine().get());

        return args;
    }

    public void volume(Action<VolumeSpec> configure) {
        getOptions().add("-v");
        getOptions().add(provider(() -> {
            var volumeSpec = getObjectFactory().newInstance(VolumeSpec.class);
            configure.execute(volumeSpec);
            return volumeSpec.resolveSpec();
        }));
    }

}
